package com.basistech.rlp.lucene;
/******************************************************************************
** This data and information is proprietary to, and a valuable trade secret
** of, Basis Technology Corp.  It is given in confidence by Basis Technology
** and may only be used as permitted under the license agreement under which
** it has been distributed, and in no other way.
**
** Copyright (c) 2007 Basis Technology Corp. All rights reserved.
**
** The technical data and information provided herein are provided with
** `limited rights', and the computer software provided herein is provided
** with `restricted rights' as those terms are defined in DAR and ASPR
** 7-104.9(a).
******************************************************************************/
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.lucene.analysis.Analyzer;

import com.basistech.util.LanguageCode;

/**
 * Utility class to create an appropriate RLP based Analyzer for the given language.
 * <br>
 * NOTE: This class is primarily for the analyzer demo programs.  
 * It may not be suitable for general use.
 */
public class RLPAnalyzerDispatcher {
    private static Logger logger = LoggerFactory.getLogger(RLPAnalyzerDispatcher.class);
    
    /**
     * Calls the default constructor of the Analyzer named RLP<i>Xx</i>Analyzer for the given language code <i>xx</i>.
     * If no such Analyzer is found, then it calls the generic {@link RLPAnalyzer}.
     * @param langCode The two letter ISO language code.  Only first two letters are inspected, 
     * so that "en_US" can also select the English Analyzer.
     * @return The newly instantiated Analyzer.
     */
    public static Analyzer createAnalyzer(String langCode){
        Analyzer ana = null;
        String analyzerClassName = "com.basistech.rlp.lucene.RLP" 
                                + langCode.substring(0, 1).toUpperCase()
                                + langCode.substring(1, 2).toLowerCase()
                                + "Analyzer";
        try{

            ana = (Analyzer) Class.forName(analyzerClassName).newInstance();
            logger.info("{} instantiated.", analyzerClassName);
        }catch (Exception e) {
            ana = new RLPAnalyzer(LanguageCode.lookupByISO639(langCode.substring(0,2)));
            logger.info("No {} found.  Constructor RLPAnalyzer(\"{}\") was used.", analyzerClassName, langCode);
        }
        return ana;
    }
    
    private RLPAnalyzerDispatcher(){} //Prevent instantiation of this class.
}
