//// -*- mode: Java; coding: utf-8 -*-
/******************************************************************************
 ** This data and information is proprietary to, and a valuable trade secret
 ** of, Basis Technology Corp.  It is given in confidence by Basis Technology
 ** and may only be used as permitted under the license agreement under which
 ** it has been distributed, and in no other way.
 **
 ** Copyright (c) 2008 Basis Technology Corp. All rights reserved.
 **
 ** The technical data and information provided herein are provided with
 ** `limited rights', and the computer software provided herein is provided
 ** with `restricted rights' as those terms are defined in DAR and ASPR
 ** 7-104.9(a).
 ******************************************************************************/
package com.basistech.rlp.solr;

import java.io.IOException;

import org.apache.lucene.analysis.TokenStream;
import org.apache.solr.analysis.BaseTokenFilterFactory;
import org.apache.solr.common.ResourceLoader;
import org.apache.solr.common.SolrException;
import org.apache.solr.util.plugin.ResourceLoaderAware;

import com.basistech.rlp.lucene.RLPPOSFilter;

/**
 * Factory for the POS (part-of-speech) Filter that passes only tokens with
 * predetermined POS tags.
 * 
 * This must be used in conjunction with RLPTokenizerFactory.
 * 
 */
public class RLPPOSFilterFactory extends BaseTokenFilterFactory implements ResourceLoaderAware{
    private String validPOSArray[] = null;

    /**
     * Specifies the list of allowed POS tags.
     * 
     * One attribute is supported in schema.xml for this factory class:
     * <ul>
     * <li>pos="<i>file-path-to-POS-list-text-file</i>"
     * </ul>
     * The file is a plain text file that lists allowed POS tags, one per line.
     * This attribute <en>must</en> be specified.
     * Valid POS tags depend on the Base Linguistic language processor that you use.
     * <br/>
     * The file path can be an absolute path, or a relative path to the solr/conf directory or the current working directory.
     * @param loader SOLR resource loader.
     * @see org.apache.solr.core.SolrResourceLoader#openResource(String)
     */
    public void inform(ResourceLoader loader) {
        String validPOSFile = args.get("pos");
        if (validPOSFile == null) {
            throw new RLPSolrException(
                    SolrException.ErrorCode.SERVER_ERROR,
                    "pos attribute must be specified to point to a file that lists part-of-speech tags that should pass.");
        }
        try {
            validPOSArray = (String[]) loader.getLines(validPOSFile).toArray(new String[0]);
        } catch (IOException e) {
            throw new RLPSolrException(SolrException.ErrorCode.SERVER_ERROR, e);
        }
    }

    /**
     * Creates a POS (part-of-speech) filter.
     * 
     * @param input The input text.
     * @return The POS filter.
     */
    public TokenStream create(TokenStream input) {
        return new RLPPOSFilter(input, validPOSArray);
    }

}
