package org.apache.lucene.demo;

/**
 * Copyright 2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Modified by Basis Technology to use the RLP-powered Analyzers.
 * BT+ indicates the new lines added by Basis Technology and,
 * BT- indicates the original lines commented out by Basis Technology.
 */
import org.apache.lucene.analysis.Analyzer;
/*
import org.apache.lucene.analysis.standard.StandardAnalyzer; // BT-
*/
import com.basistech.rlp.lucene.*; // BT+

import org.apache.lucene.document.Document;
import org.apache.lucene.index.FilterIndexReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;

import java.io.BufferedReader;
/*
import java.io.FileReader; // BT-
*/
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;

/** Simple command-line based search demo. */
public class SearchFiles {

  /** Use the norms from one field for all fields.  Norms are read into memory,
   * using a byte of memory per document per searched field.  This can cause
   * search of large collections with a large number of fields to run out of
   * memory.  If all of the fields contain only a single token, then the norms
   * are all identical, then single norm vector may be shared. */
  private static class OneNormsReader extends FilterIndexReader {
    private String field;

    public OneNormsReader(IndexReader in, String field) {
      super(in);
      this.field = field;
    }

    public byte[] norms(String field) throws IOException {
      return in.norms(this.field);
    }
  }

  private SearchFiles() {}
  
  private static void usage(){ // BT+
      System.out.println( // BT+
              "Usage: java org.apache.lucene.demo.SearchFiles [-index dir] [-field f] [-repeat n] [-queries file] [-raw] [-norms field] <lang_code>"); // BT+
      System.exit(0); // BT+
  } // BT+

  /** Simple command-line based search demo. */
  public static void main(String[] args) throws Exception {
    /*
    String usage = //BT-
       "Usage: java org.apache.lucene.demo.SearchFiles [-index dir] [-field f] [-repeat n] [-queries file] [-raw] [-norms field]"; // BT-
    */
    if (args.length > 0 && ("-h".equals(args[0]) || "-help".equals(args[0]))) {
    /*
       System.out.println(usage); // BT-
       System.exit(0); // BT-
    */
        usage(); // BT+
    }

    String index = "index";
    String field = "contents";
    String queries = null;
    int repeat = 0;
    boolean raw = false;
    String normsField = null;
    String langCode = null; // BT+
    
    for (int i = 0; i < args.length; i++) {
      if ("-index".equals(args[i])) {
        index = args[i+1];
        i++;
      } else if ("-field".equals(args[i])) {
        field = args[i+1];
        i++;
      } else if ("-queries".equals(args[i])) {
        queries = args[i+1];
        i++;
      } else if ("-repeat".equals(args[i])) {
        repeat = Integer.parseInt(args[i+1]);
        i++;
      } else if ("-raw".equals(args[i])) {
        raw = true;
      } else if ("-norms".equals(args[i])) {
        normsField = args[i+1];
        i++;
      } else { // BT+
          // Assume this is the language code.  // BT+
          if(langCode!=null){//Already set. // BT+
              usage();  // BT+
          } // BT+
          langCode = args[i]; // BT+
      } // BT+
    }
    
    if(langCode==null) usage(); // BT+
    
    IndexReader reader = IndexReader.open(index);

    if (normsField != null)
      reader = new OneNormsReader(reader, normsField);

    Searcher searcher = new IndexSearcher(reader);

    /*
    Analyzer analyzer = new StandardAnalyzer(); // BT-
	*/							
    Analyzer analyzer = RLPAnalyzerDispatcher.createAnalyzer(langCode);  // BT+


    BufferedReader in = null;
    if (queries != null) {
      /*
      in = new BufferedReader(new FileReader(queries)); // BT-
      */
      in = new BufferedReader(new InputStreamReader(new FileInputStream(queries), "UTF-8")); // BT+
    } else {
      /*
      in = new BufferedReader(new InputStreamReader(System.in, "UTF-8")); // BT-
      */ // BT: No UTF-8 for console input.  The system default encoding should be used.
      in = new BufferedReader(new InputStreamReader(System.in)); // BT+
    }
    QueryParser parser = new QueryParser(field, analyzer);
    while (true) {
      if (queries == null)                        // prompt the user
        /*
        System.out.print("Query: "); // BT-
        */ // BT: Under ant print() doesn't flush even with flush(). Use println() instead.
        System.out.println("Enter query: "); // BT+


      String line = in.readLine();

      /*
      if (line == null || line.length() == -1) // BT-
      */
      if (line == null || line.length() <= 0) // BT+
        break;

      Query query = parser.parse(line);
      System.out.println("Searching for: " + query.toString(field));

      Hits hits = searcher.search(query);
      
      if (repeat > 0) {                           // repeat & time as benchmark
        Date start = new Date();
        for (int i = 0; i < repeat; i++) {
          hits = searcher.search(query);
        }
        Date end = new Date();
        System.out.println("Time: "+(end.getTime()-start.getTime())+"ms");
      }

      System.out.println(hits.length() + " total matching documents");

      final int HITS_PER_PAGE = 10;
      for (int start = 0; start < hits.length(); start += HITS_PER_PAGE) {
        int end = Math.min(hits.length(), start + HITS_PER_PAGE);
        for (int i = start; i < end; i++) {

          if (raw) {                              // output raw format
            System.out.println("doc="+hits.id(i)+" score="+hits.score(i));
            continue;
          }

          Document doc = hits.doc(i);
          String path = doc.get("path");
          if (path != null) {
            System.out.println((i+1) + ". " + path);
            String title = doc.get("title");
            if (title != null) {
              System.out.println("   Title: " + doc.get("title"));
            }
          } else {
            System.out.println((i+1) + ". " + "No path for this document");
          }
        }

        if (queries != null)                      // non-interactive
          break;
        
        if (hits.length() > end) {
          /*
          System.out.print("more (y/n) ? "); // BT-
          */
          System.out.println("more (y/n) ? "); // BT+
          line = in.readLine();
          if (line.length() == 0 || line.charAt(0) == 'n')
            break;
        }
      }
    }
    reader.close();
  }
}
